searchData={"items":[{"type":"module","title":"Brama","doc":"Brama is a library for managing connections to external dependencies.\n\nIt provides features for:\n- Connection monitoring\n- Circuit breaking\n- Self-healing\n- Status notifications","ref":"Brama.html"},{"type":"module","title":"Connection Management - Brama","doc":"Brama allows you to register connections to external services and track their status:\n\n```elixir\n# Register a connection\nBrama.register(\"payment_api\")\n\n# Check if a connection is available\nBrama.available?(\"payment_api\")\n\n# Report success/failure\nBrama.success(\"payment_api\")\nBrama.failure(\"payment_api\", reason: \"Timeout\")\n```","ref":"Brama.html#module-connection-management"},{"type":"module","title":"Circuit Breaking - Brama","doc":"Brama implements the circuit breaker pattern to prevent cascading failures:\n\n```elixir\n# Circuit opens automatically after consecutive failures\nBrama.failure(\"payment_api\", reason: \"Timeout\") # After max_attempts, circuit opens\n\n# Manual circuit control\nBrama.open_circuit!(\"payment_api\", reason: \"Maintenance\")\nBrama.close_circuit!(\"payment_api\")\nBrama.reset_circuit!(\"payment_api\")\n```","ref":"Brama.html#module-circuit-breaking"},{"type":"module","title":"Configuration - Brama","doc":"Brama supports both global and per-connection configuration:\n\n```elixir\n# Update global settings\nBrama.configure(max_attempts: 15, expiry: 120_000)\n\n# Update specific connection\nBrama.configure(\"payment_api\", max_attempts: 5, expiry: 30_000)\n\n# Register with progressive backoff\nBrama.register(\"flaky_service\",\n  expiry_strategy: :progressive,\n  initial_expiry: 10_000,\n  max_expiry: 300_000,\n  backoff_factor: 2.0\n)\n```","ref":"Brama.html#module-configuration"},{"type":"module","title":"Event Subscription - Brama","doc":"Subscribe to events to get notified of state changes:\n\n```elixir\n# Subscribe to all events for a specific connection\nBrama.subscribe(connection: \"payment_api\")\n\n# Subscribe to specific event types\nBrama.subscribe(events: [:state_change, :failure])\n\n# Subscribe with a filter function\nBrama.subscribe(filter: fn event -> event.connection == \"payment_api\" end)\n```","ref":"Brama.html#module-event-subscription"},{"type":"function","title":"Brama.available?/2","doc":"Checks if a connection is available (circuit is closed or half-open).","ref":"Brama.html#available?/2"},{"type":"function","title":"Parameters - Brama.available?/2","doc":"- `identifier`: The connection identifier\n- `opts`: Options\n  - `:scope`: Optional scope to match","ref":"Brama.html#available?/2-parameters"},{"type":"function","title":"Examples - Brama.available?/2","doc":"```elixir\niex> Brama.register(\"payment_api\")\niex> Brama.available?(\"payment_api\")\ntrue\n```","ref":"Brama.html#available?/2-examples"},{"type":"function","title":"Brama.close_circuit!/2","doc":"Manually closes the circuit for a connection.","ref":"Brama.html#close_circuit!/2"},{"type":"function","title":"Parameters - Brama.close_circuit!/2","doc":"- `identifier`: The connection identifier\n- `opts`: Options\n  - `:scope`: Optional scope to match","ref":"Brama.html#close_circuit!/2-parameters"},{"type":"function","title":"Examples - Brama.close_circuit!/2","doc":"```elixir\niex> Brama.register(\"payment_api\")\niex> Brama.close_circuit!(\"payment_api\")\n:ok\n```","ref":"Brama.html#close_circuit!/2-examples"},{"type":"function","title":"Brama.configure/2","doc":"Updates the configuration for a specific connection or globally.","ref":"Brama.html#configure/2"},{"type":"function","title":"Parameters - Brama.configure/2","doc":"- `identifier` - Optional connection identifier. If not provided, updates global settings\n- `opts` - Configuration options to update:\n  - `:max_attempts` - Maximum number of failures before opening circuit\n  - `:expiry` - Time in milliseconds before open circuit transitions to half-open\n  - `:expiry_strategy` - Either `:fixed` or `:progressive` for backoff\n  - `:initial_expiry` - Initial expiry time for progressive backoff\n  - `:max_expiry` - Maximum expiry time for progressive backoff\n  - `:backoff_factor` - Factor for exponential backoff","ref":"Brama.html#configure/2-parameters"},{"type":"function","title":"Examples - Brama.configure/2","doc":"```elixir\n# Update global settings\nBrama.configure(max_attempts: 15, expiry: 120_000)\n\n# Update specific connection\nBrama.configure(\"payment_api\", max_attempts: 5, expiry: 30_000)\n\n# Configure progressive backoff\nBrama.configure(\"flaky_service\",\n  expiry_strategy: :progressive,\n  initial_expiry: 10_000,\n  max_expiry: 300_000,\n  backoff_factor: 2.0\n)\n```","ref":"Brama.html#configure/2-examples"},{"type":"function","title":"Brama.failure/2","doc":"Reports a failed connection attempt.","ref":"Brama.html#failure/2"},{"type":"function","title":"Parameters - Brama.failure/2","doc":"- `identifier`: The connection identifier\n- `opts`: Options\n  - `:scope`: Optional scope to match\n  - `:reason`: Reason for the failure\n  - `:metadata`: Additional metadata about the failure","ref":"Brama.html#failure/2-parameters"},{"type":"function","title":"Examples - Brama.failure/2","doc":"```elixir\niex> Brama.register(\"payment_api\")\niex> Brama.failure(\"payment_api\", reason: \"Timeout\")\n:ok\n```","ref":"Brama.html#failure/2-examples"},{"type":"function","title":"Brama.open_circuit!/2","doc":"Manually opens the circuit for a connection.","ref":"Brama.html#open_circuit!/2"},{"type":"function","title":"Parameters - Brama.open_circuit!/2","doc":"- `identifier`: The connection identifier\n- `opts`: Options\n  - `:scope`: Optional scope to match\n  - `:reason`: Reason for opening the circuit\n  - `:expiry`: Custom expiry time in milliseconds","ref":"Brama.html#open_circuit!/2-parameters"},{"type":"function","title":"Examples - Brama.open_circuit!/2","doc":"```elixir\niex> Brama.register(\"payment_api\")\niex> Brama.open_circuit!(\"payment_api\", reason: \"Maintenance\")\n:ok\n```","ref":"Brama.html#open_circuit!/2-examples"},{"type":"function","title":"Brama.register/2","doc":"Registers a new connection with Brama.","ref":"Brama.html#register/2"},{"type":"function","title":"Parameters - Brama.register/2","doc":"- `identifier`: A unique identifier for the connection\n- `opts`: Options for the connection\n  - `:scope`: Optional scope for grouping connections\n  - `:max_attempts`: Maximum number of failures before opening the circuit (default: 5)\n  - `:expiry`: Time in milliseconds after which an open circuit transitions to half-open (default: 60000)\n  - `:metadata`: Additional metadata to store with the connection","ref":"Brama.html#register/2-parameters"},{"type":"function","title":"Examples - Brama.register/2","doc":"```elixir\niex> {:ok, result} = Brama.register(\"payment_api\")\niex> Map.take(result, [:identifier, :state, :failure_count])\n%{identifier: \"payment_api\", state: :closed, failure_count: 0}\n\niex> {:ok, result} = Brama.register(\"invoice_api\", scope: \"billing\", max_attempts: 5)\niex> Map.take(result, [:identifier, :scope, :state, :failure_count])\n%{identifier: \"invoice_api\", scope: \"billing\", state: :closed, failure_count: 0}\n```","ref":"Brama.html#register/2-examples"},{"type":"function","title":"Brama.reset_circuit!/2","doc":"Resets the circuit for a connection (closes it and resets failure count).","ref":"Brama.html#reset_circuit!/2"},{"type":"function","title":"Parameters - Brama.reset_circuit!/2","doc":"- `identifier`: The connection identifier\n- `opts`: Options\n  - `:scope`: Optional scope to match","ref":"Brama.html#reset_circuit!/2-parameters"},{"type":"function","title":"Examples - Brama.reset_circuit!/2","doc":"```elixir\niex> Brama.register(\"payment_api\")\niex> Brama.reset_circuit!(\"payment_api\")\n:ok\n```","ref":"Brama.html#reset_circuit!/2-examples"},{"type":"function","title":"Brama.status/2","doc":"Gets the current status of a connection.","ref":"Brama.html#status/2"},{"type":"function","title":"Parameters - Brama.status/2","doc":"- `identifier`: The connection identifier\n- `opts`: Options\n  - `:scope`: Optional scope to match","ref":"Brama.html#status/2-parameters"},{"type":"function","title":"Examples - Brama.status/2","doc":"```elixir\niex> Brama.register(\"payment_api\")\niex> {:ok, result} = Brama.status(\"payment_api\")\niex> Map.take(result, [:state, :failure_count])\n%{state: :closed, failure_count: 0}\n```","ref":"Brama.html#status/2-examples"},{"type":"function","title":"Brama.subscribe/1","doc":"Subscribes to connection events.","ref":"Brama.html#subscribe/1"},{"type":"function","title":"Parameters - Brama.subscribe/1","doc":"- `opts`: Subscription options (at least one must be provided)\n  - `:connection`: Filter events by connection identifier\n  - `:scope`: Filter events by connection scope\n  - `:events`: List of event types to subscribe to\n  - `:filter`: Custom filter function","ref":"Brama.html#subscribe/1-parameters"},{"type":"function","title":"Examples - Brama.subscribe/1","doc":"```elixir\niex> Brama.subscribe(events: [:state_change])\n{:ok, pid}\n```","ref":"Brama.html#subscribe/1-examples"},{"type":"function","title":"Brama.success/2","doc":"Reports a successful connection attempt.","ref":"Brama.html#success/2"},{"type":"function","title":"Parameters - Brama.success/2","doc":"- `identifier`: The connection identifier\n- `opts`: Options\n  - `:scope`: Optional scope to match\n  - `:metadata`: Additional metadata about the success","ref":"Brama.html#success/2-parameters"},{"type":"function","title":"Examples - Brama.success/2","doc":"```elixir\niex> Brama.register(\"payment_api\")\niex> Brama.success(\"payment_api\")\n:ok\n```","ref":"Brama.html#success/2-examples"},{"type":"function","title":"Brama.unregister/2","doc":"Unregisters a connection from Brama.","ref":"Brama.html#unregister/2"},{"type":"function","title":"Parameters - Brama.unregister/2","doc":"- `identifier`: The connection identifier\n- `opts`: Options\n  - `:scope`: Optional scope to match","ref":"Brama.html#unregister/2-parameters"},{"type":"function","title":"Examples - Brama.unregister/2","doc":"```elixir\niex> Brama.register(\"payment_api\")\niex> Brama.unregister(\"payment_api\")\n:ok\n```","ref":"Brama.html#unregister/2-examples"},{"type":"function","title":"Brama.unsubscribe/1","doc":"Unsubscribes from connection events.","ref":"Brama.html#unsubscribe/1"},{"type":"function","title":"Parameters - Brama.unsubscribe/1","doc":"- `subscription_id`: The subscription ID returned from subscribe/1","ref":"Brama.html#unsubscribe/1-parameters"},{"type":"function","title":"Examples - Brama.unsubscribe/1","doc":"```elixir\niex> {:ok, subscription} = Brama.subscribe(events: [:state_change])\niex> Brama.unsubscribe(subscription)\n:ok\n```","ref":"Brama.html#unsubscribe/1-examples"},{"type":"module","title":"Brama.ConnectionManager","doc":"Manages connection states and implements circuit breaking logic.\n\nThis module is responsible for:\n- Tracking connection states (closed, open, half-open)\n- Implementing circuit breaking logic\n- Managing state transitions\n- Cleaning up expired connections","ref":"Brama.ConnectionManager.html"},{"type":"function","title":"Brama.ConnectionManager.available?/2","doc":"Checks if a connection is available (circuit is closed or half-open).","ref":"Brama.ConnectionManager.html#available?/2"},{"type":"function","title":"Options - Brama.ConnectionManager.available?/2","doc":"* `:scope` - Optional scope to match","ref":"Brama.ConnectionManager.html#available?/2-options"},{"type":"function","title":"Brama.ConnectionManager.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Brama.ConnectionManager.html#child_spec/1"},{"type":"function","title":"Brama.ConnectionManager.close_circuit!/2","doc":"Manually closes the circuit for a connection.","ref":"Brama.ConnectionManager.html#close_circuit!/2"},{"type":"function","title":"Options - Brama.ConnectionManager.close_circuit!/2","doc":"* `:scope` - Optional scope to match","ref":"Brama.ConnectionManager.html#close_circuit!/2-options"},{"type":"function","title":"Brama.ConnectionManager.configure/2","doc":"Updates the configuration for a specific connection or globally.","ref":"Brama.ConnectionManager.html#configure/2"},{"type":"function","title":"Parameters - Brama.ConnectionManager.configure/2","doc":"- `identifier` - Optional connection identifier. If not provided, updates global settings\n- `opts` - Configuration options to update","ref":"Brama.ConnectionManager.html#configure/2-parameters"},{"type":"type","title":"Brama.ConnectionManager.connection_data/0","doc":"","ref":"Brama.ConnectionManager.html#t:connection_data/0"},{"type":"type","title":"Brama.ConnectionManager.connection_id/0","doc":"","ref":"Brama.ConnectionManager.html#t:connection_id/0"},{"type":"type","title":"Brama.ConnectionManager.connection_scope/0","doc":"","ref":"Brama.ConnectionManager.html#t:connection_scope/0"},{"type":"type","title":"Brama.ConnectionManager.connection_state/0","doc":"","ref":"Brama.ConnectionManager.html#t:connection_state/0"},{"type":"function","title":"Brama.ConnectionManager.failure/2","doc":"Reports a failed connection attempt.","ref":"Brama.ConnectionManager.html#failure/2"},{"type":"function","title":"Options - Brama.ConnectionManager.failure/2","doc":"* `:scope` - Optional scope to match\n* `:reason` - Reason for the failure\n* `:metadata` - Additional metadata about the failure","ref":"Brama.ConnectionManager.html#failure/2-options"},{"type":"function","title":"Brama.ConnectionManager.open_circuit!/2","doc":"Manually opens the circuit for a connection.","ref":"Brama.ConnectionManager.html#open_circuit!/2"},{"type":"function","title":"Options - Brama.ConnectionManager.open_circuit!/2","doc":"* `:scope` - Optional scope to match\n* `:reason` - Reason for opening the circuit\n* `:expiry` - Custom expiry time in milliseconds","ref":"Brama.ConnectionManager.html#open_circuit!/2-options"},{"type":"function","title":"Brama.ConnectionManager.register/2","doc":"Registers a new connection.","ref":"Brama.ConnectionManager.html#register/2"},{"type":"function","title":"Options - Brama.ConnectionManager.register/2","doc":"* `:scope` - Optional scope for grouping connections\n* `:max_attempts` - Maximum number of failures before opening circuit\n* `:expiry` - Time in milliseconds after which an open circuit transitions to half-open\n* `:metadata` - Additional metadata to store with the connection","ref":"Brama.ConnectionManager.html#register/2-options"},{"type":"function","title":"Brama.ConnectionManager.reset_circuit!/2","doc":"Resets the circuit for a connection (closes it and resets failure count).","ref":"Brama.ConnectionManager.html#reset_circuit!/2"},{"type":"function","title":"Options - Brama.ConnectionManager.reset_circuit!/2","doc":"* `:scope` - Optional scope to match","ref":"Brama.ConnectionManager.html#reset_circuit!/2-options"},{"type":"function","title":"Brama.ConnectionManager.start_link/1","doc":"Starts the connection manager.","ref":"Brama.ConnectionManager.html#start_link/1"},{"type":"function","title":"Brama.ConnectionManager.status/2","doc":"Gets the current status of a connection.","ref":"Brama.ConnectionManager.html#status/2"},{"type":"function","title":"Options - Brama.ConnectionManager.status/2","doc":"* `:scope` - Optional scope to match","ref":"Brama.ConnectionManager.html#status/2-options"},{"type":"function","title":"Brama.ConnectionManager.success/2","doc":"Reports a successful connection attempt.","ref":"Brama.ConnectionManager.html#success/2"},{"type":"function","title":"Options - Brama.ConnectionManager.success/2","doc":"* `:scope` - Optional scope to match\n* `:metadata` - Additional metadata about the success","ref":"Brama.ConnectionManager.html#success/2-options"},{"type":"function","title":"Brama.ConnectionManager.unregister/2","doc":"Unregisters a connection.","ref":"Brama.ConnectionManager.html#unregister/2"},{"type":"function","title":"Options - Brama.ConnectionManager.unregister/2","doc":"* `:scope` - Optional scope to match","ref":"Brama.ConnectionManager.html#unregister/2-options"},{"type":"module","title":"Brama.Decorator","doc":"Provides macros for decorating functions with circuit breaker functionality to improve system resilience\nand handle failures gracefully.","ref":"Brama.Decorator.html"},{"type":"module","title":"Overview - Brama.Decorator","doc":"The circuit breaker pattern helps prevent cascading failures by monitoring service calls and\nautomatically stopping requests when a service is experiencing issues. It operates in three states:\n- Closed (normal operation)\n- Open (service calls blocked)\n- Half-open (testing if service recovered)","ref":"Brama.Decorator.html#module-overview"},{"type":"module","title":"Configuration - Brama.Decorator","doc":"The decorator accepts the following options:\n* `identifier` - (required) A unique string identifier for the circuit breaker instance\n* `error_handler` - (optional) A function to handle responses and determine success/failure status","ref":"Brama.Decorator.html#module-configuration"},{"type":"module","title":"Error Handling - Brama.Decorator","doc":"The error handler function can return the following statuses:\n* `:success` - Operation completed successfully, keeps circuit closed\n* `:failure` - Operation failed, increments failure count\n* `{:failure, reason}` - Operation failed with specific reason\n* `:ignore` - Operation result should not affect circuit breaker state\n\nDefault error handler behavior:\n```elixir\ndef default_error_handler(result) do\n  case result do\n    {:ok, _} -> :success\n    {:error, reason} -> {:failure, reason}\n    _ -> :ignore\n  end\nend\n```","ref":"Brama.Decorator.html#module-error-handling"},{"type":"module","title":"Exception Handling - Brama.Decorator","doc":"When an exception occurs:\n1. The exception is logged for debugging\n2. A failure is recorded with the exception message as reason\n3. The original exception is re-raised with preserved stacktrace","ref":"Brama.Decorator.html#module-exception-handling"},{"type":"module","title":"Service Unavailability - Brama.Decorator","doc":"When the circuit is open:\n1. The decorated function is not executed\n2. Returns `{:error, status}` where status contains circuit breaker state\n3. Prevents overwhelming failing services with requests","ref":"Brama.Decorator.html#module-service-unavailability"},{"type":"module","title":"Examples - Brama.Decorator","doc":"Basic usage with default error handling:\n```elixir\ndefmodule PaymentService do\n  use Brama.Decorator\n\n  @decorate circuit_breaker(identifier: \"payment_api\")\n  def process_payment(payment) do\n    PaymentAPI.process(payment)\n  end\nend\n```\n\nCustom error handling:\n```elixir\ndefmodule PaymentService do\n  use Brama.Decorator\n\n  @decorate circuit_breaker(\n    identifier: \"refund_api\",\n    error_handler: fn\n      {:ok, _} -> :success\n      {:error, :invalid_amount} -> {:failure, :validation_error}\n      {:error, :network_timeout} -> {:failure, :service_unavailable}\n      _ -> :ignore\n    end\n  )\n  def process_refund(refund) do\n    RefundAPI.process(refund)\n  end\nend\n```","ref":"Brama.Decorator.html#module-examples"},{"type":"module","title":"Return Values - Brama.Decorator","doc":"The decorated function preserves the original return value when the circuit is closed.\nWhen the circuit is open, it returns:\n```elixir\n{:error, status} # where status contains circuit breaker state information\n```","ref":"Brama.Decorator.html#module-return-values"},{"type":"macro","title":"Brama.Decorator.circuit_breaker/1","doc":"","ref":"Brama.Decorator.html#circuit_breaker/1"},{"type":"function","title":"Brama.Decorator.circuit_breaker/3","doc":"Decorator function that wraps a function with circuit breaker functionality.","ref":"Brama.Decorator.html#circuit_breaker/3"},{"type":"function","title":"Options - Brama.Decorator.circuit_breaker/3","doc":"- `identifier` - Connection identifier (required)\n- `error_handler` - Custom function to handle errors and determine success/failure","ref":"Brama.Decorator.html#circuit_breaker/3-options"},{"type":"function","title":"Examples - Brama.Decorator.circuit_breaker/3","doc":"```elixir\n@decorate circuit_breaker(identifier: \"payment_api\")\ndef process_payment(payment) do\n  # This function will be wrapped with circuit breaker logic\n  PaymentAPI.process(payment)\nend\n```","ref":"Brama.Decorator.html#circuit_breaker/3-examples"},{"type":"function","title":"Brama.Decorator.default_error_handler/1","doc":"","ref":"Brama.Decorator.html#default_error_handler/1"},{"type":"function","title":"Brama.Decorator.ensure_registered/1","doc":"","ref":"Brama.Decorator.html#ensure_registered/1"},{"type":"module","title":"Brama.EventDispatcher","doc":"Handles event dispatching and subscription management for Brama.\n\nThis module is responsible for:\n- Maintaining a registry of event subscribers\n- Filtering events based on subscriber preferences\n- Dispatching events to interested subscribers","ref":"Brama.EventDispatcher.html"},{"type":"function","title":"Brama.EventDispatcher.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Brama.EventDispatcher.html#child_spec/1"},{"type":"function","title":"Brama.EventDispatcher.dispatch/4","doc":"Dispatches an event to all matching subscribers.\n\nThis function constructs an event map and notifies all subscribers\nwhose filters match the event.","ref":"Brama.EventDispatcher.html#dispatch/4"},{"type":"function","title":"Brama.EventDispatcher.start_link/1","doc":"Starts the event dispatcher.","ref":"Brama.EventDispatcher.html#start_link/1"},{"type":"function","title":"Brama.EventDispatcher.subscribe/1","doc":"Subscribes to Brama events.","ref":"Brama.EventDispatcher.html#subscribe/1"},{"type":"function","title":"Options - Brama.EventDispatcher.subscribe/1","doc":"* `:events` - List of event types to subscribe to (default: all)\n* `:connection` - Connection identifier to filter by (default: all)\n* `:scope` - Connection scope to filter by (default: all)","ref":"Brama.EventDispatcher.html#subscribe/1-options"},{"type":"function","title":"Brama.EventDispatcher.unsubscribe/1","doc":"Unsubscribes from Brama events.","ref":"Brama.EventDispatcher.html#unsubscribe/1"},{"type":"module","title":"Brama.TestHelpers","doc":"Provides helper functions for testing applications that use Brama.\n\nThis module is intended to be used in test environments to:\n- Manipulate circuit states directly\n- Simulate failures and recovery","ref":"Brama.TestHelpers.html"},{"type":"module","title":"Usage - Brama.TestHelpers","doc":"```elixir\n# In your test file\nuse ExUnit.Case\nimport Brama.TestHelpers\n\ntest \"circuit opens after failures\" do\n  # Setup\n  Brama.register(\"test_api\")\n\n  # Simulate failures\n  add_failures(\"test_api\", 10)\n\n  # Assert circuit state\n  assert {:ok, %{state: :open}} = Brama.status(\"test_api\")\nend","ref":"Brama.TestHelpers.html#module-usage"},{"type":"function","title":"Brama.TestHelpers.add_failures/3","doc":"Adds a specific number of failures to a connection.","ref":"Brama.TestHelpers.html#add_failures/3"},{"type":"function","title":"Examples - Brama.TestHelpers.add_failures/3","doc":"iex> add_failures(\"payment_api\", 5)\n    :ok","ref":"Brama.TestHelpers.html#add_failures/3-examples"},{"type":"function","title":"Brama.TestHelpers.set_state/3","doc":"Directly sets the state of a circuit.","ref":"Brama.TestHelpers.html#set_state/3"},{"type":"function","title":"Examples - Brama.TestHelpers.set_state/3","doc":"iex> set_state(\"payment_api\", :open)\n    :ok","ref":"Brama.TestHelpers.html#set_state/3-examples"},{"type":"function","title":"Brama.TestHelpers.simulate_failures/3","doc":"Simulates a service temporarily failing.","ref":"Brama.TestHelpers.html#simulate_failures/3"},{"type":"function","title":"Examples - Brama.TestHelpers.simulate_failures/3","doc":"iex> simulate_failures(\"payment_api\", 5)\n    :ok","ref":"Brama.TestHelpers.html#simulate_failures/3-examples"},{"type":"function","title":"Brama.TestHelpers.simulate_recovery/2","doc":"Simulates a service recovering.","ref":"Brama.TestHelpers.html#simulate_recovery/2"},{"type":"function","title":"Examples - Brama.TestHelpers.simulate_recovery/2","doc":"iex> simulate_recovery(\"payment_api\")\n    :ok","ref":"Brama.TestHelpers.html#simulate_recovery/2-examples"},{"type":"extras","title":"Brama","doc":"# Brama\n\nBrama is an Elixir library for reliable connection management with external dependencies. It provides robust tracking of connection statuses to APIs, services, databases, or any external system, with built-in circuit breaking to prevent cascading failures.\n\n> This library is my first attempt at utilizing the [\"stdlib\" approach](https://ghuntley.com/stdlib/) for building software. This method involves creating comprehensive specifications before implementation, building a collection of Cursor rules to guide development, and treating AI as an autonomous agent rather than just a coding assistant. The whole library is going to be written by Cursor Agent with my supervision.","ref":"readme.html"},{"type":"extras","title":"Overview - Brama","doc":"When your application depends on external systems, knowing their availability status becomes critical. Brama serves as a gatekeeper (the name \"Brama\" means \"gate\" in several languages), monitoring your connections and protecting your application from external system failures.","ref":"readme.html#overview"},{"type":"extras","title":"Features - Brama","doc":"- **Connection Monitoring**: Track status of any connection\n- **Circuit Breaking**: Automatically prevent requests to failing systems after a threshold is reached\n- **Self-Healing**: Connections automatically reset after a configurable expiry time\n- **Status Notifications**: Subscribe to connection status change events\n- **Failure Isolation**: Protect your application from cascading failures\n- **Minimal Configuration**: Simple setup with reasonable defaults\n- **Decorator API**: Simple function decorators for automatic circuit breaking\n- **Flexible Expiry Strategies**: Fixed or progressive backoff strategies","ref":"readme.html#features"},{"type":"extras","title":"Installation - Brama","doc":"Add Brama to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:brama, \"~> 0.1.0\"},\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Circuit Breaking Mechanism - Brama","doc":"Brama implements circuit breaking for any connection with these behaviors:\n\n1. Each connection type is tracked separately\n2. After a configurable number of failed attempts (default: 10), the circuit opens\n3. When open, all requests are rejected without attempting the external call\n4. After a configurable time period (default: 1 minute), the circuit transitions to half-open\n5. Connections have immediate status updates","ref":"readme.html#circuit-breaking-mechanism"},{"type":"extras","title":"Configuration Options - Brama","doc":"```elixir\nconfig :brama,\n  max_attempts: 10,           # Attempts before circuit opens\n  cleanup_interval: 10_000,   # Status check interval in ms\n  expiry: 60_000              # Circuit open duration in ms\n```","ref":"readme.html#configuration-options"},{"type":"extras","title":"Basic Usage - Brama","doc":"```elixir\n# Register a connection\nBrama.register(\"payment_api\", type: :http, scope: \"payment_services\")\n\n# Before making an external call\nif Brama.available?(\"payment_api\") do\n  case make_api_call() do\n    {:ok, result} -> \n      Brama.success(\"payment_api\")\n      {:ok, result}\n    {:error, reason} -> \n      Brama.failure(\"payment_api\")\n      {:error, reason}\n  end\nelse\n  # Use fallback or return error\n  {:error, :service_unavailable}\nend\n```","ref":"readme.html#basic-usage"},{"type":"extras","title":"Decorator API - Brama","doc":"For a cleaner integration, use the decorator API:\n\n```elixir\ndefmodule PaymentService do\n  use Brama.Decorator\n\n  @decorate circuit_breaker(identifier: \"payment_api\")\n  def process_payment(payment) do\n    PaymentAPI.process(payment)\n  end\n  \n  @decorate circuit_breaker(\n    identifier: \"refund_api\",\n    error_handler: fn\n      {:ok, _} -> :success\n      {:error, :invalid_amount} -> {:failure, :validation_error}\n      {:error, :network_timeout} -> {:failure, :service_unavailable}\n      _ -> :ignore\n    end\n  )\n  def process_refund(refund) do\n    RefundAPI.process(refund)\n  end\nend\n```","ref":"readme.html#decorator-api"},{"type":"extras","title":"Expiry Strategies - Brama","doc":"Brama supports different strategies for determining how long a circuit remains open:","ref":"readme.html#expiry-strategies"},{"type":"extras","title":"Fixed Expiry (Default) - Brama","doc":"```elixir\nBrama.configure(\"payment_api\",\n  expiry_strategy: :fixed,  # This is the default strategy\n  expiry: 30_000            # 30 seconds\n)\n```","ref":"readme.html#fixed-expiry-default"},{"type":"extras","title":"Progressive Backoff - Brama","doc":"```elixir\nBrama.configure(\"flaky_service\",\n  expiry_strategy: :progressive,\n  initial_expiry: 5_000,     # Start with 5 seconds\n  max_expiry: 300_000,       # Cap at 5 minutes\n  backoff_factor: 2.0        # Double the time with each failure\n)\n```\n\nWith progressive backoff, expiry times would increase with each failure: 5s → 10s → 20s → 40s → etc.","ref":"readme.html#progressive-backoff"},{"type":"extras","title":"Event Notifications - Brama","doc":"Subscribe to connection status changes:\n\n```elixir\n# Subscribe to all state changes\nBrama.subscribe(events: [:state_change])\n\n# Handle events in a GenServer\ndef handle_info({:brama_event, event}, state) do\n  Logger.info(\"Connection #{event.connection} changed to #{event.data.new_state}\")\n  {:noreply, state}\nend\n```","ref":"readme.html#event-notifications"},{"type":"extras","title":"Advanced Usage - Brama","doc":"You can extend Brama for specific needs:\n\n- Create custom monitoring modules for specialized protocols\n- Implement advanced health checking logic\n- Build metrics collection around connection statuses\n- Integrate with monitoring and alerting systems","ref":"readme.html#advanced-usage"},{"type":"extras","title":"Contributing - Brama","doc":"Contributions are welcome! Please feel free to submit a Pull Request.","ref":"readme.html#contributing"},{"type":"extras","title":"License - Brama","doc":"Brama is released under the MIT License.","ref":"readme.html#license"},{"type":"extras","title":"Brama Library Specifications","doc":"# Brama Library Specifications","ref":"specs.html"},{"type":"extras","title":"Overview - Brama Library Specifications","doc":"This document serves as an index for all specification documents related to the Brama library, which provides reliable connection management with external dependencies.\n\nBrama is designed to act as a gatekeeper for your application's interactions with external systems. It monitors connection statuses, implements circuit breaking to prevent cascading failures, and provides self-healing mechanisms and notifications for connection status changes.","ref":"specs.html#overview"},{"type":"extras","title":"Specifications Index - Brama Library Specifications","doc":"| Domain Area | Description | Link |\n|-------------|-------------|------|\n| Architecture | Overall system architecture and design principles | [Architecture](specs/architecture.md) |\n| Connection Monitoring | Specifications for tracking connection status | [Connection Monitoring](specs/connection_monitoring.md) |\n| Circuit Breaking | How the circuit breaker mechanism prevents cascading failures | [Circuit Breaking](specs/circuit_breaking.md) |\n| Self-Healing | Automatic recovery mechanisms for failed connections | [Self-Healing](specs/self_healing.md) |\n| Status Notifications | Event system for connection status changes | [Status Notifications](specs/status_notifications.md) |\n| Failure Isolation | Techniques to isolate failures and protect the application | [Failure Isolation](specs/failure_isolation.md) |\n| Configuration | Configuration options and defaults | [Configuration](specs/configuration.md) |\n| Testing Strategy | Approaches for testing the library | [Testing Strategy](specs/testing_strategy.md) |\n| API | Public interface and usage patterns (includes Decorator API) | [API](specs/api.md) |","ref":"specs.html#specifications-index"},{"type":"extras","title":"Development Roadmap - Brama Library Specifications","doc":"This specification will evolve as development progresses. Implementation priorities:\n\n1. Core connection monitoring and registry functionality\n2. Basic circuit breaking mechanism\n3. Expiry strategies (fixed and progressive)\n4. Status notification system\n5. Function decorator API for seamless integration\n6. Event subscription and delivery system\n7. Telemetry integration\n8. Advanced configuration options\n9. Documentation and examples\n\nThe development will follow an iterative approach, starting with the core functionality and progressively adding features while maintaining backward compatibility.","ref":"specs.html#development-roadmap"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.37.3"}}